use polars::prelude::*;
use rand::Rng;
use rand_distr::{Distribution, Normal};

use crate::types::{ColumnSchema, DataType};

pub fn generate_dataframe_from_schema(num_rows: usize, schema: Vec<ColumnSchema>) -> PolarsResult<DataFrame> {
    let mut columns = Vec::new();
    
    for column_schema in schema {
        let series = match column_schema.dtype {
            DataType::Boolean => generate_boolean_series(&column_schema.name, num_rows),
            DataType::Int32 => generate_int32_series(&column_schema.name, num_rows),
            DataType::Int64 => generate_int64_series(&column_schema.name, num_rows),
            DataType::Float32 => generate_float32_series(&column_schema.name, num_rows),
            DataType::Float64 => generate_float64_series(&column_schema.name, num_rows),
            DataType::Utf8 => generate_string_series(&column_schema.name, num_rows),
            DataType::Date => generate_date_series(&column_schema.name, num_rows),
            DataType::DateTime => generate_datetime_series(&column_schema.name, num_rows),
            DataType::Time => generate_time_series(&column_schema.name, num_rows),
            DataType::Other => generate_null_series(&column_schema.name, num_rows),
        };
        columns.push(series);
    }
    
    DataFrame::new(columns)
}

fn generate_boolean_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let values: Vec<Option<bool>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(rng.gen::<bool>())
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_int32_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let normal = Normal::new(0.0, 1000.0).unwrap();
    let values: Vec<Option<i32>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(normal.sample(&mut rng) as i32)
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_int64_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let normal = Normal::new(0.0, 1000.0).unwrap();
    let values: Vec<Option<i64>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(normal.sample(&mut rng) as i64)
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_float32_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let normal = Normal::new(0.0, 1000.0).unwrap();
    let values: Vec<Option<f32>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(normal.sample(&mut rng) as f32)
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_float64_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let normal = Normal::new(0.0, 1000.0).unwrap();
    let values: Vec<Option<f64>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(normal.sample(&mut rng))
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_string_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let words = vec!["apple", "banana", "cherry", "date", "elderberry", "fig", "grape"];
    let values: Vec<Option<String>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(words[rng.gen_range(0..words.len())].to_string())
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_date_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let base_date = chrono::NaiveDate::from_ymd_opt(2020, 1, 1).unwrap();
    let values: Vec<Option<chrono::NaiveDate>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(base_date + chrono::Duration::days(rng.gen_range(0..1000)))
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_datetime_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let base_datetime = chrono::NaiveDateTime::from_timestamp_opt(1577836800, 0).unwrap(); // 2020-01-01 00:00:00
    let values: Vec<Option<chrono::NaiveDateTime>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                Some(base_datetime + chrono::Duration::seconds(rng.gen_range(0..31536000))) // within 1 year
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_time_series(name: &str, num_rows: usize) -> Series {
    let mut rng = rand::thread_rng();
    let values: Vec<Option<chrono::NaiveTime>> = (0..num_rows)
        .map(|_| {
            if rng.gen::<f64>() < 0.1 {
                None // 10% null values
            } else {
                chrono::NaiveTime::from_hms_opt(
                    rng.gen_range(0..24),
                    rng.gen_range(0..60),
                    rng.gen_range(0..60),
                )
            }
        })
        .collect();
    Series::new(name, values)
}

fn generate_null_series(name: &str, num_rows: usize) -> Series {
    let values: Vec<Option<i32>> = vec![None; num_rows];
    Series::new(name, values)
}