#![allow(non_local_definitions)]

mod types;
mod generate;

use types::{ColumnSchema, DataType, LogicalType};
use generate::generate_dataframe_from_schema;
use polars::prelude::*;
use pyo3::prelude::*;
use pyo3_polars::{PyDataFrame, PySeries};
use strum::IntoEnumIterator;

#[pyclass]
#[derive(Clone)]
pub struct PyColumnSchema {
    #[pyo3(get, set)]
    pub id: String,
    #[pyo3(get, set)]
    pub name: String,
    #[pyo3(get, set)]
    pub dtype: PyDataType,
    #[pyo3(get, set)]
    pub logical_type: PyLogicalType,
}

#[pyclass]
#[derive(Clone)]
pub enum PyDataType {
    Boolean,
    Int32,
    Int64,
    Float32,
    Float64,
    Date,
    DateTime,
    Time,
    Utf8,
    Other,
}

#[pyclass]
#[derive(Clone)]
pub enum PyLogicalType {
    Boolean,
    Categorical,
    Date,
    DateTime,
    Id,
    NaturalLanguage,
    Number,
    Other,
    Time,
    Null,
}

#[pymethods]
impl PyColumnSchema {
    #[new]
    fn new(id: String, name: String, dtype: PyDataType, logical_type: PyLogicalType) -> Self {
        Self {
            id,
            name,
            dtype,
            logical_type,
        }
    }
}

impl From<PyDataType> for DataType {
    fn from(p: PyDataType) -> Self {
        match p {
            PyDataType::Boolean => DataType::Boolean,
            PyDataType::Int32 => DataType::Int32,
            PyDataType::Int64 => DataType::Int64,
            PyDataType::Float32 => DataType::Float32,
            PyDataType::Float64 => DataType::Float64,
            PyDataType::Date => DataType::Date,
            PyDataType::DateTime => DataType::DateTime,
            PyDataType::Time => DataType::Time,
            PyDataType::Utf8 => DataType::Utf8,
            PyDataType::Other => DataType::Other,
        }
    }
}

impl From<PyLogicalType> for LogicalType {
    fn from(p: PyLogicalType) -> Self {
        match p {
            PyLogicalType::Boolean => LogicalType::Boolean,
            PyLogicalType::Categorical => LogicalType::Categorical,
            PyLogicalType::Date => LogicalType::Date,
            PyLogicalType::DateTime => LogicalType::DateTime,
            PyLogicalType::Id => LogicalType::Id,
            PyLogicalType::NaturalLanguage => LogicalType::NaturalLanguage,
            PyLogicalType::Number => LogicalType::Number,
            PyLogicalType::Other => LogicalType::Other,
            PyLogicalType::Time => LogicalType::Time,
            PyLogicalType::Null => LogicalType::Null,
        }
    }
}

impl From<PyColumnSchema> for ColumnSchema {
    fn from(p: PyColumnSchema) -> Self {
        ColumnSchema {
            id: p.id,
            name: p.name,
            dtype: p.dtype.into(),
            logical_type: p.logical_type.into(),
        }
    }
}

#[pyfunction]
fn add(a: i32, b: i32) -> i32 {
    a + b
}

#[pyfunction]
fn make_series() -> PySeries {
    let series = Series::new("values", &[1, 2, 3, 4]);
    PySeries(series)
}

#[pyfunction]
fn make_df() -> PyDataFrame {
    let mut schema = vec![];
    for (i, variant) in DataType::iter().enumerate() {
        schema.push(ColumnSchema {
            id: format!("{}", i),
            name: format!("Col{}-{}", i, variant.to_string()),
            dtype: variant,
            logical_type: LogicalType::Boolean,
        })
    }

    let df = generate_dataframe_from_schema(200, schema).unwrap();

    PyDataFrame(df)
}

#[pyfunction]
fn make_custom_df(schema: Vec<PyColumnSchema>, rows: usize) -> PyResult<PyDataFrame> {
    let core_schema: Vec<ColumnSchema> = schema.into_iter().map(Into::into).collect();
    let df = generate_dataframe_from_schema(rows, core_schema)
        .map_err(|e| PyErr::new::<pyo3::exceptions::PyValueError, _>(e.to_string()))?;
    Ok(PyDataFrame(df))
}

#[pymodule]
fn feature_synth(_py: Python, m: &PyModule) -> PyResult<()> {
    m.add_class::<PyColumnSchema>()?;
    m.add_class::<PyDataType>()?;
    m.add_class::<PyLogicalType>()?;

    m.add_function(wrap_pyfunction!(add, m)?)?;
    m.add_function(wrap_pyfunction!(make_series, m)?)?;
    m.add_function(wrap_pyfunction!(make_df, m)?)?;
    m.add_function(wrap_pyfunction!(make_custom_df, m)?)?;
    Ok(())
}
